Package: Colossus
Type: Package
Title: "Risk Model Regression and Analysis with Complex Non-Linear
        Models"
Version: 1.5.1
Authors@R: c(
    person("Eric", "King-Giunta", email="egiunta@ksu.edu", role=c("aut", "cre"), comment = c(ORCID = "0000-0002-1577-766X")),
    person("Amir", "Bahadori", role=c("ctb"), comment = c(ORCID = "0000-0002-4589-105X")),
    person("Dan", "Andresen", role=c("ctb"), comment = c(ORCID = "0000-0003-2345-6695")),
    person("Linda", "Walsh", role=c("ctb"), comment = c(ORCID = "0000-0001-7399-9191")),
    person("Benjamin", "French", role=c("ctb"), comment = c(ORCID = "0000-0001-9265-5378")),
    person("Lawrence", "Dauer", role=c("ctb"), comment = c(ORCID = "0000-0002-5629-8462")),
    person("John", "Boice Jr", role=c("ctb"), comment = c(ORCID = "0000-0002-8755-1299")),
    person("Kansas State University", role=c("cph")),
    person("NASA", role=c("fnd")),
    person("NCRP", role=c("fnd")),
    person("NRC", role=c("fnd"))
    )
URL: https://ericgiunta.github.io/Colossus/,
        https://github.com/ericgiunta/Colossus
BugReports: https://github.com/ericgiunta/Colossus/issues
Description: Performs survival analysis using general non-linear models. Risk models can be the sum or product of terms. Each term is the product of exponential/linear functions of covariates. Additionally sub-terms can be defined as a sum of exponential, linear threshold, and step functions. Cox Proportional hazards <https://en.wikipedia.org/wiki/Proportional_hazards_model>, Poisson <https://en.wikipedia.org/wiki/Poisson_regression>, and Fine-Gray competing risks <https://www.publichealth.columbia.edu/research/population-health-methods/competing-risk-analysis> regression are supported. This work was sponsored by NASA Grants 80NSSC19M0161 and 80NSSC23M0129 through a subcontract from the National Council on Radiation Protection and Measurements (NCRP). The computing for this project was performed on the Beocat Research Cluster at Kansas State University, which is funded in part by NSF grants CNS-1006860, EPS-1006860, EPS-0919443, ACI-1440548, CHE-1726332, and NIH P20GM113109.
License: GPL (>= 3)
Imports: Rcpp, data.table, parallel, stats, utils, rlang, methods,
        callr, stringr, processx, dplyr, tibble, lubridate
LinkingTo: Rcpp, RcppEigen, testthat
SystemRequirements: make, C++17
RoxygenNote: 7.3.3
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat, xml2, pandoc, spelling, survival,
        splines, ggplot2
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
Biarch: TRUE
NeedsCompilation: yes
Packaged: 2026-04-21 14:17:51 UTC; user
Author: Eric King-Giunta [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1577-766X>),
  Amir Bahadori [ctb] (ORCID: <https://orcid.org/0000-0002-4589-105X>),
  Dan Andresen [ctb] (ORCID: <https://orcid.org/0000-0003-2345-6695>),
  Linda Walsh [ctb] (ORCID: <https://orcid.org/0000-0001-7399-9191>),
  Benjamin French [ctb] (ORCID: <https://orcid.org/0000-0001-9265-5378>),
  Lawrence Dauer [ctb] (ORCID: <https://orcid.org/0000-0002-5629-8462>),
  John Boice Jr [ctb] (ORCID: <https://orcid.org/0000-0002-8755-1299>),
  Kansas State University [cph],
  NASA [fnd],
  NCRP [fnd],
  NRC [fnd]
Maintainer: Eric King-Giunta <egiunta@ksu.edu>
Repository: CRAN
Date/Publication: 2026-04-22 20:30:22 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-23 00:13:51 UTC; unix
Archs: Colossus.so.dSYM
