# Generated by roxygen2: do not edit by hand

S3method(EventAssignment,default)
S3method(EventAssignment,poisres)
S3method(EventAssignment,poisresbound)
S3method(LikelihoodBound,coxres)
S3method(LikelihoodBound,default)
S3method(LikelihoodBound,poisres)
S3method(RelativeRisk,coxres)
S3method(RelativeRisk,default)
S3method(Residual,default)
S3method(Residual,poisres)
S3method(plot,coxres)
S3method(plotMartingale,coxres)
S3method(plotMartingale,default)
S3method(plotRisk,coxres)
S3method(plotRisk,default)
S3method(plotSchoenfeld,coxres)
S3method(plotSchoenfeld,default)
S3method(plotSurvival,coxres)
S3method(plotSurvival,default)
S3method(print,caseconres)
S3method(print,coxres)
S3method(print,coxresbound)
S3method(print,logitres)
S3method(print,poisres)
S3method(print,poisresbound)
export(CaseControlRun)
export(ColossusCoxSurv)
export(ColossusLogitSurv)
export(ColossusPoisSurv)
export(CoxRun)
export(CoxRunMulti)
export(Date_Shift)
export(EventAssignment)
export(Event_Count_Gen)
export(Event_Time_Gen)
export(Joint_Multiple_Events)
export(LikelihoodBound)
export(Likelihood_Ratio_Test)
export(Linked_Dose_Formula)
export(Linked_Lin_Exp_Para)
export(LogisticRun)
export(PoisRun)
export(PoisRunJoint)
export(PoisRunMulti)
export(RelativeRisk)
export(Replace_Missing)
export(Residual)
export(System_Version)
export(Time_Since)
export(factorize)
export(gen_time_dep)
export(get_form)
export(get_form_joint)
export(plotMartingale)
export(plotRisk)
export(plotSchoenfeld)
export(plotSurvival)
importFrom(Rcpp,evalCpp)
importFrom(callr,rcmd)
importFrom(data.table,":=")
importFrom(data.table,.SD)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,set)
importFrom(data.table,setDT)
importFrom(data.table,setDTthreads)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(lubridate,as.duration)
importFrom(lubridate,interval)
importFrom(lubridate,make_date)
importFrom(methods,is)
importFrom(parallel,detectCores)
importFrom(processx,run)
importFrom(rlang,.data)
importFrom(stats,approxfun)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,runif)
importFrom(stats,time)
importFrom(stats,weighted.mean)
importFrom(stringr,str_count)
importFrom(stringr,str_match)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,combn)
importFrom(utils,head)
useDynLib(Colossus, .registration = TRUE)
