% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BasicRun.R
\name{LikelihoodBound.default}
\alias{LikelihoodBound.default}
\title{Generic likelihood boundary calculation function, default option}
\usage{
\method{LikelihoodBound}{default}(x, df, curve_control = list(), control = list(), ...)
}
\arguments{
\item{x}{result object from a regression, class coxres or poisres}

\item{df}{a data.table containing the columns of interest}

\item{curve_control}{a list of control options for the likelihood boundary regression. See the \code{vignette("Control_Options")} vignette for details.}

\item{control}{list of parameters controlling the convergence, see the \code{vignette("Control_Options")} vignette for details}

\item{...}{extended for other necessary parameters}
}
\description{
\code{LikelihoodBound} Generic likelihood boundary calculation function, by default nothing happens
}
