% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BasicRun.R
\name{RelativeRisk}
\alias{RelativeRisk}
\title{Generic relative risk calculation function}
\usage{
RelativeRisk(x, df, ...)
}
\arguments{
\item{x}{result object from a regression, class coxres}

\item{df}{a data.table containing the columns of interest}

\item{...}{extended for other necessary parameters}
}
\description{
\code{RelativeRisk} Generic relative risk calculation function
}
